@@ -31,6 +31,14 @@
 
 #define IVAN_VERSION "0.50"
 
+class item;
+class material;
+class character;
+
+typedef const item citem;
+typedef const material cmaterial;
+typedef const character ccharacter;
+
 struct databasebase
 {
   int Config;
@@ -116,7 +124,7 @@ const name##prototype name::ProtoType
 #define BLOATED 5
 #define OVER_FED 6
 
-#define STATES 20
+#define STATES 21
 
 #define POLYMORPHED (1 << 0)
 #define HASTE (1 << 1)
@@ -160,8 +168,11 @@ const name##prototype name::ProtoType
 #define ENERGY 32
 #define POISON 64
 #define DRAIN 128
+#define MUSTARD_GAS_DAMAGE 256
+#define PSI 512
 #define THROW 32768
 
+
 #define UNDEFINED 0
 #define MALE 1
 #define FEMALE 2
@@ -211,6 +222,7 @@ const name##prototype name::ProtoType
 #define DEFINITE 2
 #define INDEFINE_BIT 4
 #define INDEFINITE 6
+#define STRIPPED 8
 
 #define TRANSPARENT_COLOR 0xF81F // pink
 
@@ -224,11 +236,12 @@ const name##prototype name::ProtoType
 #define GR_EFFECT 5
 #define GR_CURSOR 6
 
-#define GRAPHIC_TYPES 3
+#define GRAPHIC_TYPES 4
 
 #define GR_WTERRAIN 0
 #define GR_FOW 1
 #define GR_SYMBOL 2
+#define GR_SMILEY 3
 
 /* SpecialFlags for graphics system. No one knows what "ST_" means... */
 
@@ -259,7 +272,7 @@ const name##prototype name::ProtoType
 #define WOBBLE_FREQ_SHIFT 4
 #define WOBBLE_FREQ_RANGE (3 << WOBBLE_FREQ_SHIFT)
 
-const v2 SILHOUETTE_SIZE(48, 64);
+cv2 SILHOUETTE_SIZE(48, 64);
 
 #define ITEM_CATEGORIES 18
 
@@ -360,6 +373,8 @@ const v2 SILHOUETTE_SIZE(48, 64);
 #define EFFECT_OMMEL_TEARS 27
 #define EFFECT_OMMEL_SNOT 28
 #define EFFECT_OMMEL_BONE 29
+#define EFFECT_MUSTARD_GAS 30
+#define EFFECT_MUSTARD_GAS_LIQUID 31
 
 /* CEM = Consume End Message */
 
@@ -403,25 +418,27 @@ const v2 SILHOUETTE_SIZE(48, 64);
 #define USE_LEGS 2
 #define USE_HEAD 4
 
-#define ATTRIBUTES 10
-#define BASE_ATTRIBUTES 6
+#define ATTRIBUTES 11
+#define BASE_ATTRIBUTES 7
 
 #define ENDURANCE 0
 #define PERCEPTION 1
 #define INTELLIGENCE 2
 #define WISDOM 3
-#define CHARISMA 4
-#define MANA 5
+#define WILL_POWER 4
+#define CHARISMA 5
+#define MANA 6
 
-#define ARM_STRENGTH 6
-#define LEG_STRENGTH 7
-#define DEXTERITY 8
-#define AGILITY 9
+#define ARM_STRENGTH 7
+#define LEG_STRENGTH 8
+#define DEXTERITY 9
+#define AGILITY 10
 
 #define F_ENDURANCE (1 << ENDURANCE)
 #define F_PERCEPTION (1 << PERCEPTION)
 #define F_INTELLIGENCE (1 << INTELLIGENCE)
 #define F_WISDOM (1 << WISDOM)
+#define F_WILL_POWER (1 << WILL_POWER)
 #define F_CHARISMA (1 << CHARISMA)
 #define F_MANA (1 << MANA)
 
@@ -644,6 +661,7 @@ const v2 SILHOUETTE_SIZE(48, 64);
 
 #define HEXAGONAL_LOCK (S_LOCK_ID + LOCK_DELTA * 1)
 #define OCTAGONAL_LOCK (S_LOCK_ID + LOCK_DELTA * 2)
+#define HEART_SHAPED_LOCK (S_LOCK_ID + LOCK_DELTA * 3)
 
 #define DESERT 1
 #define JUNGLE 2
@@ -852,8 +870,10 @@ const v2 SILHOUETTE_SIZE(48, 64);
 #define RED_CURSOR 1
 #define BLUE_CURSOR 2
 #define YELLOW_CURSOR 3
-#define FLASH 0x4000
-#define TARGET 0x8000
+#define CURSOR_FLASH 0x2000
+#define CURSOR_TARGET 0x4000
+#define CURSOR_BIG 0x8000
+#define CURSOR_FLAGS (CURSOR_BIG|CURSOR_FLASH|CURSOR_TARGET)
 
 #define GRAY_FRACTAL 0
 #define RED_FRACTAL 1
@@ -1049,6 +1069,7 @@ const v2 SILHOUETTE_SIZE(48, 64);
 #define IS_SPARKLING 8
 #define IS_SCARY 16
 #define IS_GOLEM_MATERIAL 32
+#define IS_BEVERAGE 64
 
 /* BodyFlags */
 #define IS_ALIVE 1
@@ -1063,13 +1084,14 @@ const v2 SILHOUETTE_SIZE(48, 64);
 #define CAN_DISSOLVE 4
 #define AFFECT_INSIDE 8
 #define EFFECT_IS_GOOD 16
+#define IS_AFFECTED_BY_MUSTARD_GAS 32
 
 /*************************/
 /* End of DataBase flags */
 /*************************/
 
 #define TILE_SIZE 16
-const v2 TILE_V2(TILE_SIZE, TILE_SIZE);
+cv2 TILE_V2(TILE_SIZE, TILE_SIZE);
 
 #define SQUARE_INDEX_MASK 0xFFFF
 #define ALLOW_ANIMATE 0x10000
@@ -1088,4 +1110,12 @@ const v2 TILE_V2(TILE_SIZE, TILE_SIZE);
 
 #define NO_MONSTER_GENERATION 1
 
+#define NO_TAMING -1
+
+#define SADIST_HIT 1
+
+#define EQUIPMENT_DATAS 13
+
+#define SPECIAL_CONFIGURATION_GENERATION_LEVELS 2
+
 #endif
