@@ -50,8 +50,8 @@
 #include "balance.h"
 #include "confdef.h"
 
-#define SAVE_FILE_VERSION 118 // Increment this if changes make savefiles incompatible
-#define BONE_FILE_VERSION 105 // Increment this if changes make bonefiles incompatible
+#define SAVE_FILE_VERSION 119 // Increment this if changes make savefiles incompatible
+#define BONE_FILE_VERSION 106 // Increment this if changes make bonefiles incompatible
 
 #define LOADED 0
 #define NEW_GAME 1
@@ -115,19 +115,20 @@ character* game::Petrus = 0;
 time_t game::TimePlayedBeforeLastLoad;
 time_t game::LastLoad;
 time_t game::GameBegan;
+truth game::PlayerHasReceivedAllGodsKnownBonus;
 
 festring game::AutoSaveFileName = game::GetSaveDir() + "AutoSave";
-const char* const game::Alignment[] = { "L++", "L+", "L", "L-", "N+", "N=", "N-", "C+", "C", "C-", "C--" };
+cchar* const game::Alignment[] = { "L++", "L+", "L", "L-", "N+", "N=", "N-", "C+", "C", "C-", "C--" };
 god** game::God;
 
-const int game::MoveNormalCommandKey[] = { KEY_HOME, KEY_UP, KEY_PAGE_UP, KEY_LEFT, KEY_RIGHT, KEY_END, KEY_DOWN, KEY_PAGE_DOWN, '.' };
-const int game::MoveAbnormalCommandKey[] = { '7','8','9','u','o','j','k','l','.' };
+cint game::MoveNormalCommandKey[] = { KEY_HOME, KEY_UP, KEY_PAGE_UP, KEY_LEFT, KEY_RIGHT, KEY_END, KEY_DOWN, KEY_PAGE_DOWN, '.' };
+cint game::MoveAbnormalCommandKey[] = { '7','8','9','u','o','j','k','l','.' };
 
-const v2 game::MoveVector[] = { v2(-1, -1), v2(0, -1), v2(1, -1), v2(-1, 0), v2(1, 0), v2(-1, 1), v2(0, 1), v2(1, 1), v2(0, 0) };
-const v2 game::RelativeMoveVector[] = { v2(-1, -1), v2(1, 0), v2(1, 0), v2(-2, 1), v2(2, 0), v2(-2, 1), v2(1, 0), v2(1, 0), v2(-1, -1) };
-const v2 game::BasicMoveVector[] = { v2(-1, 0), v2(1, 0), v2(0, -1), v2(0, 1) };
-const v2 game::LargeMoveVector[] = { v2(-1, -1), v2(0, -1), v2(1, -1), v2(2, -1), v2(-1, 0), v2(2, 0), v2(-1, 1), v2(2, 1), v2(-1, 2), v2(0, 2), v2(1, 2), v2(2, 2), v2(0, 0), v2(1, 0), v2(0, 1), v2(1, 1) };
-const int game::LargeMoveDirection[] = { 0, 1, 1, 2, 3, 4, 3, 4, 5, 6, 6, 7, 8, 8, 8, 8 };
+cv2 game::MoveVector[] = { v2(-1, -1), v2(0, -1), v2(1, -1), v2(-1, 0), v2(1, 0), v2(-1, 1), v2(0, 1), v2(1, 1), v2(0, 0) };
+cv2 game::RelativeMoveVector[] = { v2(-1, -1), v2(1, 0), v2(1, 0), v2(-2, 1), v2(2, 0), v2(-2, 1), v2(1, 0), v2(1, 0), v2(-1, -1) };
+cv2 game::BasicMoveVector[] = { v2(-1, 0), v2(1, 0), v2(0, -1), v2(0, 1) };
+cv2 game::LargeMoveVector[] = { v2(-1, -1), v2(0, -1), v2(1, -1), v2(2, -1), v2(-1, 0), v2(2, 0), v2(-1, 1), v2(2, 1), v2(-1, 2), v2(0, 2), v2(1, 2), v2(2, 2), v2(0, 0), v2(1, 0), v2(0, 1), v2(1, 1) };
+cint game::LargeMoveDirection[] = { 0, 1, 1, 2, 3, 4, 3, 4, 5, 6, 6, 7, 8, 8, 8, 8 };
 
 truth game::LOSUpdateRequested = false;
 uchar*** game::LuxTable = 0;
@@ -161,11 +162,15 @@ bitmap* game::BusyAnimationCache[32];
 festring game::PlayerName;
 ulong game::EquipmentMemory[MAX_EQUIPMENT_SLOTS];
 olterrain* game::MonsterPortal;
+std::vector<v2> game::SpecialCursorPos;
+std::vector<int> game::SpecialCursorData;
+cbitmap* game::EnterImage;
+v2 game::EnterTextDisplacement;
 
 void game::AddCharacterID(character* Char, ulong ID) {
   if(CharacterIDMap.find(ID) != CharacterIDMap.end())
     int esko = esko = 2;
-  CharacterIDMap.insert(std::pair<ulong, character*>(ID, Char));
+  CharacterIDMap.insert(std::make_pair(ID, Char));
 }
 void game::RemoveCharacterID(ulong ID) {
   if(CharacterIDMap.find(ID) == CharacterIDMap.end())
@@ -175,7 +180,7 @@ void game::RemoveCharacterID(ulong ID) {
 void game::AddItemID(item* Item, ulong ID) {
   if(ItemIDMap.find(ID) != ItemIDMap.end())
     int esko = esko = 2;
-  ItemIDMap.insert(std::pair<ulong, item*>(ID, Item));
+  ItemIDMap.insert(std::make_pair(ID, Item));
 }
 void game::RemoveItemID(ulong ID)
 {
@@ -194,7 +199,7 @@ void game::AddTrapID(entity* Trap, ulong ID) {
     int esko = esko = 2;
 
   if(ID)
-    TrapIDMap.insert(std::pair<ulong, entity*>(ID, Trap));
+    TrapIDMap.insert(std::make_pair(ID, Trap));
 }
 void game::RemoveTrapID(ulong ID)
 {
@@ -220,7 +225,7 @@ void game::InitScript()
   GameScript->RandomizeLevels();
 }
 
-truth game::Init(const festring& Name)
+truth game::Init(cfestring& Name)
 {
   if(Name.IsEmpty())
     if(ivanconfig::GetDefaultName().IsEmpty())
@@ -237,24 +242,18 @@ truth game::Init(const festring& Name)
 
 #ifdef WIN32
   _mkdir("Save");
-#endif
-
-#ifdef __DJGPP__
-  mkdir("Save", S_IWUSR);
-#endif
-
-#ifdef LINUX
-  mkdir(GetSaveDir().CStr(), S_IRWXU|S_IRWXG);
-#endif
-
-#ifdef WIN32
   _mkdir("Bones");
 #endif
 
 #ifdef __DJGPP__
+  mkdir("Save", S_IWUSR);
   mkdir("Bones", S_IWUSR);
 #endif
 
+#ifdef LINUX
+  mkdir(GetSaveDir().CStr(), S_IRWXU|S_IRWXG);
+#endif
+
   LOSTick = 2;
   DangerFound = 0;
   CausePanicFlag = false;
@@ -307,7 +306,7 @@ truth game::Init(const festring& Name)
       globalwindowhandler::InstallControlLoop(AnimationController);
       SetIsRunning(true);
       InWilderness = true;
-      iosystem::TextScreen(CONST_S("Generating game...\n\nThis may take some time, please wait."), WHITE, false, &BusyAnimation);
+      iosystem::TextScreen(CONST_S("Generating game...\n\nThis may take some time, please wait."), ZERO_V2, WHITE, false, true, &BusyAnimation);
       igraph::CreateBackGround(GRAY_FRACTAL);
       NextCharacterID = 1;
       NextItemID = 1;
@@ -370,6 +369,7 @@ truth game::Init(const festring& Name)
       GameBegan = time(0);
       LastLoad = time(0);
       TimePlayedBeforeLastLoad = time::GetZeroTime();
+      bool PlayerHasReceivedAllGodsKnownBonus = false;
       ADD_MESSAGE("You commence your journey to Attnam. Use direction keys to move, '>' to enter an area and '?' to view other commands.");
 
       if(IsXMas())
@@ -654,7 +654,7 @@ void game::UpdateCameraCoordinate(int& Coordinate, int Center, int Size, int Scr
     GetCurrentArea()->SendNewDrawRequest();
 }
 
-const char* game::Insult() // convert to array
+cchar* game::Insult() // convert to array
 {
   switch(RAND_N(18))
   {
@@ -682,7 +682,7 @@ const char* game::Insult() // convert to array
 
 /* DefaultAnswer = REQUIRES_ANSWER the question requires an answer */
 
-truth game::TruthQuestion(const festring& String, int DefaultAnswer, int OtherKeyForTrue)
+truth game::TruthQuestion(cfestring& String, int DefaultAnswer, int OtherKeyForTrue)
 {
   if(DefaultAnswer == NO)
     DefaultAnswer = 'n';
@@ -720,6 +720,12 @@ void game::DrawEverythingNoBlit(truth AnimationDraw)
     else
       DOUBLE_BUFFER->Fill(CalculateScreenCoordinates(CursorPos), TILE_V2, 0);
 
+  int c;
+
+  for(c = 0; c < SpecialCursorPos.size(); ++c)
+    if(OnScreen(SpecialCursorPos[c]))
+      CurrentArea->GetSquare(SpecialCursorPos[c])->SendStrongNewDrawRequest();
+
   globalwindowhandler::UpdateTick();
   GetCurrentArea()->Draw(AnimationDraw);
   Player->DrawPanel(AnimationDraw);
@@ -729,10 +735,10 @@ void game::DrawEverythingNoBlit(truth AnimationDraw)
 
   if(OnScreen(CursorPos))
   {
-    v2 ScreenCordinates = CalculateScreenCoordinates(CursorPos);
+    v2 ScreenCoord = CalculateScreenCoordinates(CursorPos);
     blitdata B = { DOUBLE_BUFFER,
 		   { 0, 0 },
-		   { ScreenCordinates.X, ScreenCordinates.Y },
+		   { ScreenCoord.X, ScreenCoord.Y },
 		   { TILE_SIZE, TILE_SIZE },
 		   { 0 },
 		   TRANSPARENT_COLOR,
@@ -755,19 +761,44 @@ void game::DrawEverythingNoBlit(truth AnimationDraw)
       DOUBLE_BUFFER->StretchBlit(B);
     }
 
-    igraph::DrawCursor(ScreenCordinates, CursorData);
+    igraph::DrawCursor(ScreenCoord, CursorData);
   }
 
   if(Player->IsEnabled())
-  {
-    v2 Pos = Player->GetPos();
+    if(Player->IsSmall())
+    {
+      v2 Pos = Player->GetPos();
 
-    if(OnScreen(Pos))
-      igraph::DrawCursor(CalculateScreenCoordinates(Pos), Player->GetCursorData());
-  }
+      if(OnScreen(Pos))
+      {
+	v2 ScreenCoord = CalculateScreenCoordinates(Pos);
+	igraph::DrawCursor(ScreenCoord, Player->GetCursorData());
+      }
+    }
+    else
+    {
+      for(c = 0; c < Player->GetSquaresUnder(); ++c)
+      {
+	v2 Pos = Player->GetPos(c);
+
+	if(OnScreen(Pos))
+	{
+	  v2 ScreenCoord = CalculateScreenCoordinates(Pos);
+	  igraph::DrawCursor(ScreenCoord, Player->GetCursorData()|CURSOR_BIG, c);
+	}
+      }
+    }
+
+  for(c = 0; c < SpecialCursorPos.size(); ++c)
+    if(OnScreen(SpecialCursorPos[c]))
+    {
+      v2 ScreenCoord = CalculateScreenCoordinates(SpecialCursorPos[c]);
+      igraph::DrawCursor(ScreenCoord, SpecialCursorData[c]);
+      GetCurrentArea()->GetSquare(SpecialCursorPos[c])->SendStrongNewDrawRequest();
+    }
 }
 
-truth game::Save(const festring& SaveName)
+truth game::Save(cfestring& SaveName)
 {
   outputfile SaveFile(SaveName + ".sav");
   SaveFile << int(SAVE_FILE_VERSION);
@@ -813,11 +844,12 @@ truth game::Save(const festring& SaveName)
   SaveFile << GetTimeSpent();
   /* or in more readable format: time() - LastLoad + TimeAtLastLoad */
 
+  SaveFile << PlayerHasReceivedAllGodsKnownBonus;
   protosystem::SaveCharacterDataBaseFlags(SaveFile);
   return true;
 }
 
-int game::Load(const festring& SaveName)
+int game::Load(cfestring& SaveName)
 {
   inputfile SaveFile(SaveName + ".sav", 0, false);
 
@@ -891,13 +923,13 @@ int game::Load(const festring& SaveName)
   SaveFile >> DefaultPolymorphTo >> DefaultSummonMonster;
   SaveFile >> DefaultWish >> DefaultChangeMaterial >> DefaultDetectMaterial;
   SaveFile >> TimePlayedBeforeLastLoad;
+  SaveFile >> PlayerHasReceivedAllGodsKnownBonus;
   LastLoad = time(0);
-
   protosystem::LoadCharacterDataBaseFlags(SaveFile);
   return LOADED;
 }
 
-festring game::SaveName(const festring& Base)
+festring game::SaveName(cfestring& Base)
 {
   festring SaveName = GetSaveDir();
 
@@ -993,7 +1025,7 @@ void game::ShowLevelMessage()
   CurrentLevel->SetLevelMessage("");
 }
 
-int game::DirectionQuestion(const festring& Topic, truth RequireAnswer, truth AcceptYourself)
+int game::DirectionQuestion(cfestring& Topic, truth RequireAnswer, truth AcceptYourself)
 {
   for(;;)
   {
@@ -1102,7 +1134,7 @@ void game::DoEvilDeed(int Amount)
   }
 }
 
-void game::SaveWorldMap(const festring& SaveName, truth DeleteAfterwards)
+void game::SaveWorldMap(cfestring& SaveName, truth DeleteAfterwards)
 {
   outputfile SaveFile(SaveName + ".wm");
   SaveFile << WorldMap;
@@ -1114,7 +1146,7 @@ void game::SaveWorldMap(const festring& SaveName, truth DeleteAfterwards)
   }
 }
 
-worldmap* game::LoadWorldMap(const festring& SaveName)
+worldmap* game::LoadWorldMap(cfestring& SaveName)
 {
   inputfile SaveFile(SaveName + ".wm");
   SaveFile >> WorldMap;
@@ -1156,7 +1188,7 @@ void game::CreateTeams()
     for(uint c = 0; c < i->second.GetRelation().size(); ++c)
       GetTeam(i->second.GetRelation()[c].first)->SetRelation(GetTeam(i->first), i->second.GetRelation()[c].second);
 
-    const int* KillEvilness = i->second.GetKillEvilness();
+    cint* KillEvilness = i->second.GetKillEvilness();
 
     if(KillEvilness)
       GetTeam(i->first)->SetKillEvilness(*KillEvilness);
@@ -1165,7 +1197,7 @@ void game::CreateTeams()
 
 /* v2 Pos should be removed from xxxQuestion()s? */
 
-festring game::StringQuestion(const festring& Topic, col16 Color, festring::sizetype MinLetters, festring::sizetype MaxLetters, truth AllowExit, stringkeyhandler KeyHandler)
+festring game::StringQuestion(cfestring& Topic, col16 Color, festring::sizetype MinLetters, festring::sizetype MaxLetters, truth AllowExit, stringkeyhandler KeyHandler)
 {
   DrawEverythingNoBlit();
   igraph::BlitBackGround(v2(16, 6), v2(GetScreenXSize() << 4, 23)); // pos may be incorrect!
@@ -1175,7 +1207,7 @@ festring game::StringQuestion(const festring& Topic, col16 Color, festring::size
   return Return;
 }
 
-long game::NumberQuestion(const festring& Topic, col16 Color, truth ReturnZeroOnEsc)
+long game::NumberQuestion(cfestring& Topic, col16 Color, truth ReturnZeroOnEsc)
 {
   DrawEverythingNoBlit();
   igraph::BlitBackGround(v2(16, 6), v2(GetScreenXSize() << 4, 23));
@@ -1184,7 +1216,7 @@ long game::NumberQuestion(const festring& Topic, col16 Color, truth ReturnZeroOn
   return Return;
 }
 
-long game::ScrollBarQuestion(const festring& Topic, long BeginValue, long Step, long Min, long Max, long AbortValue, col16 TopicColor, col16 Color1, col16 Color2, void (*Handler)(long))
+long game::ScrollBarQuestion(cfestring& Topic, long BeginValue, long Step, long Min, long Max, long AbortValue, col16 TopicColor, col16 Color1, col16 Color2, void (*Handler)(long))
 {
   DrawEverythingNoBlit();
   igraph::BlitBackGround(v2(16, 6), v2(GetScreenXSize() << 4, 23));
@@ -1260,7 +1292,7 @@ int game::GetDirectionForVector(v2 Vector)
   return DIR_ERROR;
 }
 
-const char* game::GetVerbalPlayerAlignment()
+cchar* game::GetVerbalPlayerAlignment()
 {
   long Sum = 0;
 
@@ -1301,25 +1333,41 @@ void game::CreateGods()
 
 void game::BusyAnimation()
 {
-  BusyAnimation(DOUBLE_BUFFER);
+  BusyAnimation(DOUBLE_BUFFER, false);
 }
 
-void game::BusyAnimation(bitmap* Buffer)
+void game::BusyAnimation(bitmap* Buffer, truth ForceDraw)
 {
   static clock_t LastTime = 0;
   static int Frame = 0;
-  static blitdata B = { 0,
-			{ 0, 0 },
-			{ (RES.X >> 1) - 100, (RES.Y << 1) / 3 - 100 },
-			{ 200, 200 },
-			{ 0 },
-			0,
-			0 };
+  static blitdata B1 = { 0,
+			 { 0, 0 },
+			 { 0, 0 },
+			 { RES.X, RES.Y },
+			 { 0 },
+			 0,
+			 0 };
+  static blitdata B2 = { 0,
+			 { 0, 0 },
+			 { (RES.X >> 1) - 100, (RES.Y << 1) / 3 - 100 },
+			 { 200, 200 },
+			 { 0 },
+			 0,
+			 0 };
 
-  if(clock() - LastTime > CLOCKS_PER_SEC / 25)
+  if(ForceDraw || clock() - LastTime > CLOCKS_PER_SEC / 25)
   {
-    B.Bitmap = Buffer;
-    BusyAnimationCache[Frame]->NormalBlit(B);
+    B2.Bitmap = Buffer;
+    B2.Dest.X = (RES.X >> 1) - 100 + EnterTextDisplacement.X;
+    B2.Dest.Y = (RES.Y << 1) / 3 - 100 + EnterTextDisplacement.Y;
+
+    if(EnterImage)
+    {
+      B1.Bitmap = Buffer;
+      EnterImage->NormalMaskedBlit(B1);
+    }
+
+    BusyAnimationCache[Frame]->NormalBlit(B2);
 
     if(Buffer == DOUBLE_BUFFER)
       graphics::BlitDBToScreen();
@@ -1373,7 +1421,7 @@ void game::CreateBusyAnimationCache()
   }
 }
 
-int game::AskForKeyPress(const festring& Topic)
+int game::AskForKeyPress(cfestring& Topic)
 {
   DrawEverythingNoBlit();
   FONT->Printf(DOUBLE_BUFFER, v2(16, 8), WHITE, "%s", Topic.CapitalizeCopy().CStr());
@@ -1387,7 +1435,7 @@ int game::AskForKeyPress(const festring& Topic)
  * KeyHandler is called when the key has NOT been identified as a movement key
  * Both can be deactivated by passing 0 as parameter */
 
-v2 game::PositionQuestion(const festring& Topic, v2 CursorPos, void (*Handler)(v2), positionkeyhandler KeyHandler, truth Zoom)
+v2 game::PositionQuestion(cfestring& Topic, v2 CursorPos, void (*Handler)(v2), positionkeyhandler KeyHandler, truth Zoom)
 {
   int Key = 0;
   SetDoZoom(Zoom);
@@ -1536,14 +1584,14 @@ void game::InitGlobalValueMap()
       ABORT("Illegal datafile define on line %ld!", SaveFile.TellLine());
 
     SaveFile.ReadWord(Word);
-    GlobalValueMap.insert(std::pair<festring, long>(Word, SaveFile.ReadNumber()));
+    GlobalValueMap.insert(std::make_pair(Word, SaveFile.ReadNumber()));
   }
 }
 
-void game::TextScreen(const festring& Text, col16 Color, truth GKey, void (*BitmapEditor)(bitmap*))
+void game::TextScreen(cfestring& Text, v2 Displacement, col16 Color, truth GKey, truth Fade, bitmapeditor BitmapEditor)
 {
   globalwindowhandler::DisableControlLoops();
-  iosystem::TextScreen(Text, Color, GKey, BitmapEditor);
+  iosystem::TextScreen(Text, Displacement, Color, GKey, Fade, BitmapEditor);
   globalwindowhandler::EnableControlLoops();
 }
 
@@ -1552,7 +1600,7 @@ void game::TextScreen(const festring& Text, col16 Color, truth GKey, void (*Bitm
    Not surprisingly KeyNumber is the number of keys at ...
 */
 
-int game::KeyQuestion(const festring& Message, int DefaultAnswer, int KeyNumber, ...)
+int game::KeyQuestion(cfestring& Message, int DefaultAnswer, int KeyNumber, ...)
 {
   int* Key = new int[KeyNumber];
   va_list Arguments;
@@ -1702,7 +1750,7 @@ int game::CalculateRoughDirection(v2 Vector)
     return 4;
 }
 
-int game::Menu(bitmap* BackGround, v2 Pos, const festring& Topic, const festring& sMS, col16 Color, const festring& SmallText1, const festring& SmallText2)
+int game::Menu(bitmap* BackGround, v2 Pos, cfestring& Topic, cfestring& sMS, col16 Color, cfestring& SmallText1, cfestring& SmallText2)
 {
   globalwindowhandler::DisableControlLoops();
   int Return = iosystem::Menu(BackGround, Pos, Topic, sMS, Color, SmallText1, SmallText2);
@@ -1879,7 +1927,7 @@ void game::EnterArea(charactervector& Group, int Area, int EntryIndex)
     }
 
     GetCurrentLevel()->FiatLux();
-    const truth* AutoReveal = GetCurrentLevel()->GetLevelScript()->AutoReveal();
+    ctruth* AutoReveal = GetCurrentLevel()->GetLevelScript()->AutoReveal();
 
     if(New && AutoReveal && *AutoReveal)
       GetCurrentLevel()->Reveal();
@@ -2131,7 +2179,7 @@ ulong game::CreateNewCharacterID(character* NewChar)
   if(CharacterIDMap.find(ID) != CharacterIDMap.end())
     int esko = esko = 2;
 
-  CharacterIDMap.insert(std::pair<ulong, character*>(ID, NewChar));
+  CharacterIDMap.insert(std::make_pair(ID, NewChar));
   return ID;
 }
 
@@ -2143,7 +2191,7 @@ ulong game::CreateNewItemID(item* NewItem)
     int esko = esko = 2;
 
   if(NewItem)
-    ItemIDMap.insert(std::pair<ulong, item*>(ID, NewItem));
+    ItemIDMap.insert(std::make_pair(ID, NewItem));
 
   return ID;
 }
@@ -2156,7 +2204,7 @@ ulong game::CreateNewTrapID(entity* NewTrap)
     int esko = esko = 2;
 
   if(NewTrap)
-    TrapIDMap.insert(std::pair<ulong, entity*>(ID, NewTrap));
+    TrapIDMap.insert(std::make_pair(ID, NewTrap));
 
   return ID;
 }
@@ -2181,7 +2229,7 @@ entity* game::SearchTrap(ulong ID)
 
 outputfile& operator<<(outputfile& SaveFile, const configid& Value)
 {
-  SaveFile.Write(reinterpret_cast<const char*>(&Value), sizeof(Value));
+  SaveFile.Write(reinterpret_cast<cchar*>(&Value), sizeof(Value));
   return SaveFile;
 }
 
@@ -2263,7 +2311,7 @@ int game::GetLevels()
   return GetCurrentDungeon()->GetLevels();
 }
 
-void game::SignalDeath(const character* Ghost, const character* Murderer, festring DeathMsg)
+void game::SignalDeath(ccharacter* Ghost, ccharacter* Murderer, festring DeathMsg)
 {
   if(InWilderness)
     DeathMsg << " in the world map";
@@ -2298,7 +2346,7 @@ void game::SignalDeath(const character* Ghost, const character* Murderer, festri
 
   if(i == MassacreMap->end())
   {
-    i = MassacreMap->insert(std::pair<massacreid, killdata>(MI, killdata(1, Ghost->GetGenerationDanger()))).first;
+    i = MassacreMap->insert(std::make_pair(MI, killdata(1, Ghost->GetGenerationDanger()))).first;
     i->second.Reason.push_back(killreason(DeathMsg, 1));
   }
   else
@@ -2339,7 +2387,7 @@ struct massacresetentry
   int ImageKey;
 };
 
-void game::DisplayMassacreList(const massacremap& MassacreMap, const char* Reason, long Amount)
+void game::DisplayMassacreList(const massacremap& MassacreMap, cchar* Reason, long Amount)
 {
   std::set<massacresetentry> MassacreSet;
   festring FirstPronoun;
@@ -2435,6 +2483,8 @@ void game::DisplayMassacreList(const massacremap& MassacreMap, const char* Reaso
   }
 
   List.AddDescription(SideTopic);
+  List.AddDescription(CONST_S(""));
+  List.AddDescription("Choose a type of creatures to browse death details.");
   std::set<massacresetentry>::const_iterator i2;
 
   for(i2 = MassacreSet.begin(); i2 != MassacreSet.end(); ++i2)
@@ -3023,7 +3073,7 @@ ulong game::IncreaseSquarePartEmitationTicks()
   return SquarePartEmitationTick;
 }
 
-void game::Wish(character* Wisher, const char* MsgSingle, const char* MsgPair)
+void game::Wish(character* Wisher, cchar* MsgSingle, cchar* MsgPair)
 {
   for(;;)
   {
@@ -3171,7 +3221,7 @@ truth DistanceOrderer(character* C1, character* C2)
     return Pos1.X < Pos2.X;
 }
 
-truth game::FillPetVector(const char* Verb)
+truth game::FillPetVector(cchar* Verb)
 {
   PetVector.clear();
   team* Team = GetTeam(PLAYER_TEAM);
@@ -3252,7 +3302,7 @@ void game::PetHandler(v2 CursorPos)
   character* Char = CurrentArea->GetSquare(CursorPos)->GetCharacter();
 
   if(Char && Char->CanBeSeenByPlayer() && Char->IsPet() && !Char->IsPlayer())
-    CursorData = RED_CURSOR|TARGET;
+    CursorData = RED_CURSOR|CURSOR_TARGET;
   else
     CursorData = RED_CURSOR;
 
@@ -3300,9 +3350,9 @@ truth game::SelectPet(int Key)
   return false;
 }
 
-void game::CommandScreen(const festring& Topic, ulong PossibleFlags, ulong ConstantFlags, ulong& VaryFlags, ulong& Flags)
+void game::CommandScreen(cfestring& Topic, ulong PossibleFlags, ulong ConstantFlags, ulong& VaryFlags, ulong& Flags)
 {
-  static const char* CommandDescription[COMMAND_FLAGS] =
+  static cchar* CommandDescription[COMMAND_FLAGS] =
     {
       "Follow me",
       "Flee from enemies",
@@ -3463,9 +3513,9 @@ void game::InitAttributeMemory()
 void game::TeleportHandler(v2 CursorPos)
 {
   if((CursorPos - Player->GetPos()).GetLengthSquare() > Player->GetTeleportRangeSquare())
-    CursorData = BLUE_CURSOR|TARGET;
+    CursorData = BLUE_CURSOR|CURSOR_TARGET;
   else
-    CursorData = RED_CURSOR|TARGET;
+    CursorData = RED_CURSOR|CURSOR_TARGET;
 }
 
 double game::GetGameSituationDanger()
@@ -3565,3 +3615,70 @@ truth game::PlayerIsRunning()
 {
   return PlayerRunning && Player->CanMove();
 }
+
+void game::AddSpecialCursor(v2 Pos, int Data)
+{
+  SpecialCursorPos.push_back(Pos);
+  SpecialCursorData.push_back(Data);
+}
+
+void game::RemoveSpecialCursors()
+{
+  SpecialCursorPos.clear();
+  SpecialCursorData.clear();
+}
+
+void game::LearnAbout(god* Who)
+{
+  Who->SetIsKnown(true);
+
+  /* slightly slow, but doesn't matter since
+     this is run so rarely */
+
+  if(PlayerKnowsAllGods() && !game::PlayerHasReceivedAllGodsKnownBonus)
+  {
+    GetPlayer()->ApplyAllGodsKnownBonus();
+    game::PlayerHasReceivedAllGodsKnownBonus = true;
+  }
+}
+
+truth game::PlayerKnowsAllGods()
+{
+  for(int c = 1; c <= GODS; ++c)
+    if(!GetGod(c)->IsKnown())
+      return false;
+
+  return true;
+}
+
+void game::AdjustRelationsToAllGods(int Amount)
+{
+  for(int c = 1; c <= GODS; ++c)
+    GetGod(c)->AdjustRelation(Amount);
+}
+
+void game::ShowDeathSmiley(bitmap* Buffer, truth)
+{
+  static blitdata B = { 0,
+			{ 0, 0 },
+			{ (RES.X >> 1) - 24, RES.Y * 4 / 7 - 24 },
+			{ 48, 48 },
+			{ 0 },
+			TRANSPARENT_COLOR,
+			0 };
+
+  int Tick = globalwindowhandler::UpdateTick();  
+
+  if(((Tick >> 1) & 31) == 1)
+    B.Src.X = 48;
+  else if(((Tick >> 1) & 31) == 2)
+    B.Src.X = 96;
+  else
+    B.Src.X = 0;
+
+  B.Bitmap = Buffer;
+  igraph::GetSmileyGraphic()->NormalBlit(B);
+
+  if(Buffer == DOUBLE_BUFFER)
+    graphics::BlitDBToScreen();
+}
