@@ -23,18 +23,22 @@ truth nonhumanoid::UseMaterialAttributes() const { return GetTorso()->UseMateria
 
 truth elpuri::SpecialEnemySightedReaction(character*) { return !(Active = true); }
 
-const char* billswill::FirstPersonBiteVerb() const { return "emit psi waves at"; }
-const char* billswill::FirstPersonCriticalBiteVerb() const { return "emit powerful psi waves at"; }
-const char* billswill::ThirdPersonBiteVerb() const { return "emits psi waves at"; }
-const char* billswill::ThirdPersonCriticalBiteVerb() const { return "emits powerful psi waves at"; }
+cchar* billswill::FirstPersonBiteVerb() const { return "emit psi waves at"; }
+cchar* billswill::FirstPersonCriticalBiteVerb() const { return "emit powerful psi waves at"; }
+cchar* billswill::ThirdPersonBiteVerb() const { return "emits psi waves at"; }
+cchar* billswill::ThirdPersonCriticalBiteVerb() const { return "emits powerful psi waves at"; }
 int billswill::GetBodyPartWobbleData(int) const { return WOBBLE_HORIZONTALLY|(2 << WOBBLE_FREQ_SHIFT); }
 
 int mommo::GetBodyPartWobbleData(int) const { return (GetConfig() == CONICAL ? WOBBLE_HORIZONTALLY : WOBBLE_VERTICALLY)|(2 << WOBBLE_FREQ_SHIFT); }
 
 bodypart* dog::MakeBodyPart(int) const { return dogtorso::Spawn(0, NO_MATERIALS); }
 
+bodypart* spider::MakeBodyPart(int) const { return spidertorso::Spawn(0, NO_MATERIALS); }
+
 int dolphin::GetSpecialBodyPartFlags(int) const { return RAND() & (MIRROR|ROTATE); }
 
+bodypart* bat::MakeBodyPart(int) const { return battorso::Spawn(0, NO_MATERIALS); }
+
 col16 chameleon::GetSkinColor() const { return MakeRGB16(60 + RAND() % 190, 60 + RAND() % 190, 60 + RAND() % 190); }
 
 void floatingeye::SetWayPoints(const fearray<packv2>& What) { ArrayToVector(What, WayPoints); }
@@ -44,17 +48,17 @@ int eddy::GetBodyPartWobbleData(int) const { return WOBBLE_VERTICALLY|(2 << WOBB
 
 bodypart* magicmushroom::MakeBodyPart(int) const { return magicmushroomtorso::Spawn(0, NO_MATERIALS); }
 
-const char* ghost::FirstPersonBiteVerb() const { return "touch"; }
-const char* ghost::FirstPersonCriticalBiteVerb() const { return "awfully touch"; }
-const char* ghost::ThirdPersonBiteVerb() const { return "touches"; }
-const char* ghost::ThirdPersonCriticalBiteVerb() const { return "awfully touches"; }
+cchar* ghost::FirstPersonBiteVerb() const { return "touch"; }
+cchar* ghost::FirstPersonCriticalBiteVerb() const { return "awfully touch"; }
+cchar* ghost::ThirdPersonBiteVerb() const { return "touches"; }
+cchar* ghost::ThirdPersonCriticalBiteVerb() const { return "awfully touches"; }
 truth ghost::SpecialEnemySightedReaction(character*) { return !(Active = true); }
 int ghost::GetBodyPartWobbleData(int) const { return WOBBLE_HORIZONTALLY|(2 << WOBBLE_FREQ_SHIFT); }
 
-const char* magpie::FirstPersonBiteVerb() const { return "peck"; }
-const char* magpie::FirstPersonCriticalBiteVerb() const { return "critically peck"; }
-const char* magpie::ThirdPersonBiteVerb() const { return "pecks"; }
-const char* magpie::ThirdPersonCriticalBiteVerb() const { return "critically peck"; }
+cchar* magpie::FirstPersonBiteVerb() const { return "peck"; }
+cchar* magpie::FirstPersonCriticalBiteVerb() const { return "critically peck"; }
+cchar* magpie::ThirdPersonBiteVerb() const { return "pecks"; }
+cchar* magpie::ThirdPersonCriticalBiteVerb() const { return "critically pecks"; }
 
 bodypart* largecreature::MakeBodyPart(int) const { return largetorso::Spawn(0, NO_MATERIALS); }
 lsquare* largecreature::GetNeighbourLSquare(int I) const { return static_cast<lsquare*>(GetNeighbourSquare(I)); }
@@ -72,7 +76,7 @@ bodypart* mysticfrog::MakeBodyPart(int) const { return mysticfrogtorso::Spawn(0,
 
 bodypart* lobhse::MakeBodyPart(int) const { return lobhsetorso::Spawn(0, NO_MATERIALS); }
 
-truth elpuri::Hit(character* Enemy, v2, int, truth ForceHit)
+truth elpuri::Hit(character* Enemy, v2, int, int Flags)
 {
   if(CheckIfTooScaredToHit(Enemy))
     return false;
@@ -99,7 +103,7 @@ truth elpuri::Hit(character* Enemy, v2, int, truth ForceHit)
 
 	  if(!Abort)
 	  {
-	    nonhumanoid::Hit(ByStander, Square->GetPos(), YOURSELF, ForceHit);
+	    nonhumanoid::Hit(ByStander, Square->GetPos(), YOURSELF, Flags);
 	    ByStander->DamageAllItems(this, RAND() % 36 + RAND() % 36, PHYSICAL_DAMAGE);
 	    EnemyHit[EnemiesHit++] = ByStander;
 	  }
@@ -248,15 +252,17 @@ void nonhumanoid::Kick(lsquare* Square, int Direction, truth ForceHit)
   }
 }
 
-truth nonhumanoid::Hit(character* Enemy, v2 HitPos, int Direction, truth ForceHit)
+truth nonhumanoid::Hit(character* Enemy, v2 HitPos, int Direction, int Flags)
 {
   if(CheckIfTooScaredToHit(Enemy))
     return false;
 
-  if(IsPlayer() && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
-    return false;
-
-  if(!IsPlayer() && GetAttribute(WISDOM) >= Enemy->GetAttackWisdomLimit())
+  if(IsPlayer())
+  {
+    if(!(Enemy->IsMasochist() && GetRelation(Enemy) == FRIEND) && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
+      return false;
+  }
+  else if(GetAttribute(WISDOM) >= Enemy->GetAttackWisdomLimit())
     return false;
 
   if(GetBurdenState() == OVER_LOADED)
@@ -299,19 +305,19 @@ truth nonhumanoid::Hit(character* Enemy, v2 HitPos, int Direction, truth ForceHi
    case USE_ARMS:
     msgsystem::EnterBigMessageMode();
     Hostility(Enemy);
-    UnarmedHit(Enemy, HitPos, Direction, ForceHit);
+    UnarmedHit(Enemy, HitPos, Direction, Flags & SADIST_HIT);
     msgsystem::LeaveBigMessageMode();
     return true;
    case USE_LEGS:
     msgsystem::EnterBigMessageMode();
     Hostility(Enemy);
-    Kick(GetNearLSquare(HitPos), Direction, ForceHit);
+    Kick(GetNearLSquare(HitPos), Direction, Flags & SADIST_HIT);
     msgsystem::LeaveBigMessageMode();
     return true;
    case USE_HEAD:
     msgsystem::EnterBigMessageMode();
     Hostility(Enemy);
-    Bite(Enemy, HitPos, Direction, ForceHit);
+    Bite(Enemy, HitPos, Direction, Flags & SADIST_HIT);
     msgsystem::LeaveBigMessageMode();
     return true;
    default:
@@ -340,7 +346,7 @@ void nonhumanoid::UnarmedHit(character* Enemy, v2 HitPos, int Direction, truth F
 
 /* Returns the average number of APs required to kill Enemy */
 
-double nonhumanoid::GetTimeToKill(const character* Enemy, truth UseMaxHP) const
+double nonhumanoid::GetTimeToKill(ccharacter* Enemy, truth UseMaxHP) const
 {
   double Effectivity = 0;
   int AttackStyles = 0;
@@ -429,7 +435,7 @@ void nonhumanoid::EditExperience(int Identifier, double Value, double Speed)
 
       if(Change)
       {
-	const char* Adj = Change > 0 ? "stronger" : "weaker";
+	cchar* Adj = Change > 0 ? "stronger" : "weaker";
 
 	if(IsPlayer())
 	  ADD_MESSAGE("Your feel %s!", Adj);
@@ -451,7 +457,7 @@ void nonhumanoid::EditExperience(int Identifier, double Value, double Speed)
 
       if(Change)
       {
-	const char* Adj = Change > 0 ? "very agile" : "sluggish";
+	cchar* Adj = Change > 0 ? "very agile" : "sluggish";
 
 	if(IsPlayer())
 	  ADD_MESSAGE("Your feel %s!", Adj);
@@ -513,7 +519,7 @@ void dog::BeTalkedTo()
     if(GetRelation(PLAYER) != HOSTILE)
     {
       static truth Last;
-      const char* Reply;
+      cchar* Reply;
 
       if(GetHP() << 1 > GetMaxHP())
 	Reply = Last ? "barks happily" : "wags its tail happily";
@@ -827,7 +833,7 @@ void floatingeye::GetAICommand()
   EditAP(-1000);
 }
 
-truth floatingeye::Hit(character* Enemy, v2, int, truth)
+truth floatingeye::Hit(character* Enemy, v2, int, int)
 {
   if(IsPlayer())
     ADD_MESSAGE("You stare at %s.", Enemy->CHAR_DESCRIPTION(DEFINITE));
@@ -904,10 +910,13 @@ int chameleon::TakeHit(character* Enemy, item* Weapon, bodypart* EnemyBodyPart,
   return Return;
 }
 
-truth eddy::Hit(character* Enemy, v2, int, truth)
+truth eddy::Hit(character* Enemy, v2, int, int)
 {
-  if(IsPlayer() && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
-    return false;
+  if(IsPlayer())
+  {
+    if(!(Enemy->IsMasochist() && GetRelation(Enemy) == FRIEND) && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
+      return false;
+  }
 
   Hostility(Enemy);
 
@@ -1037,15 +1046,17 @@ void mushroom::SetSpecies(int What)
   UpdatePictures();
 }
 
-truth twoheadedmoose::Hit(character* Enemy, v2 HitPos, int Direction, truth ForceHit)
+truth twoheadedmoose::Hit(character* Enemy, v2 HitPos, int Direction, int Flags)
 {
   if(CheckIfTooScaredToHit(Enemy))
     return false;
 
-  if(IsPlayer() && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
-    return false;
-
-  if(!IsPlayer() && GetAttribute(WISDOM) >= Enemy->GetAttackWisdomLimit())
+  if(IsPlayer())
+  {
+    if(!(Enemy->IsMasochist() && GetRelation(Enemy) == FRIEND) && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
+      return false;
+  }
+  else if(GetAttribute(WISDOM) >= Enemy->GetAttackWisdomLimit())
     return false;
 
   if(GetBurdenState() == OVER_LOADED)
@@ -1058,7 +1069,7 @@ truth twoheadedmoose::Hit(character* Enemy, v2 HitPos, int Direction, truth Forc
 
   Hostility(Enemy);
   msgsystem::EnterBigMessageMode();
-  Bite(Enemy, HitPos, Direction, ForceHit);
+  Bite(Enemy, HitPos, Direction, Flags & SADIST_HIT);
   v2 Pos[MAX_NEIGHBOUR_SQUARES];
   character* Char[MAX_NEIGHBOUR_SQUARES];
   int Index = 0;
@@ -1082,7 +1093,7 @@ truth twoheadedmoose::Hit(character* Enemy, v2 HitPos, int Direction, truth Forc
   if(Index)
   {
     int ChosenIndex = RAND() % Index;
-    Bite(Char[ChosenIndex], Pos[ChosenIndex], game::GetDirectionForVector(Pos[ChosenIndex] - GetPos()), ForceHit);
+    Bite(Char[ChosenIndex], Pos[ChosenIndex], game::GetDirectionForVector(Pos[ChosenIndex] - GetPos()), Flags & SADIST_HIT);
   }
 
   msgsystem::LeaveBigMessageMode();
@@ -1423,7 +1434,7 @@ truth largecreature::IsFreeForMe(square* Square) const
   {
     v2 SquarePos = Pos + game::GetLargeMoveVector(12 + c);
 
-    if(!Area->IsValidPos(SquarePos) || (Area->GetSquare(SquarePos)->GetCharacter() && Area->GetSquare(SquarePos)->GetCharacter() != static_cast<const character*>(this)))
+    if(!Area->IsValidPos(SquarePos) || (Area->GetSquare(SquarePos)->GetCharacter() && Area->GetSquare(SquarePos)->GetCharacter() != static_cast<ccharacter*>(this)))
       return false;
   }
 
@@ -1593,6 +1604,7 @@ void hedgehog::SpecialBodyDefenceEffect(character* Enemy, bodypart* BodyPart, in
       ADD_MESSAGE("%s spines jab %s!", CHAR_POSSESSIVE_PRONOUN, Enemy->CHAR_NAME(DEFINITE));
 
     Enemy->ReceiveBodyPartDamage(this, 1 + (RAND() & 1), PHYSICAL_DAMAGE, BodyPart->GetBodyPartIndex(), YOURSELF, false, false, true, false);
+    Enemy->CheckDeath(CONST_S("killed by the pointy spines of ") + GetName(INDEFINITE), this);
   }
 }
 
@@ -1825,12 +1837,17 @@ truth largecreature::PlaceIsIllegal(v2 Pos, v2 Illegal) const
   return false;
 }
 
-truth mommo::Hit(character* Enemy, v2 Pos, int, truth)
+truth mommo::Hit(character* Enemy, v2 Pos, int, int)
 {
   if(CheckIfTooScaredToHit(Enemy))
     return false;
 
-  if(IsPlayer() && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
+  if(IsPlayer())
+  {
+    if(!(Enemy->IsMasochist() && GetRelation(Enemy) == FRIEND) && GetRelation(Enemy) != HOSTILE && !game::TruthQuestion(CONST_S("This might cause a hostile reaction. Are you sure? [y/N]")))
+      return false;
+  }
+  else if(GetAttribute(WISDOM) >= Enemy->GetAttackWisdomLimit())
     return false;
 
   Hostility(Enemy);
@@ -1897,7 +1914,7 @@ truth blinkdog::SpecialEnemySightedReaction(character*)
   return false;
 }
 
-void blinkdog::MonsterTeleport(const char* BarkMsg)
+void blinkdog::MonsterTeleport(cchar* BarkMsg)
 {
   if(CanBeSeenByPlayer())
     ADD_MESSAGE("You hear a %s inside your head as %s vanishes!", BarkMsg, CHAR_NAME(DEFINITE));
@@ -1943,14 +1960,14 @@ int blinkdog::TakeHit(character* Enemy, item* Weapon, bodypart* EnemyBodyPart, v
 
     if((RAND() & 1 && StateIsActivated(PANIC))
        || (!(RAND() & 3) && IsInBadCondition())
-       || !(RAND() & 15));
+       || !(RAND() & 15))
     MonsterTeleport("terrified yelp");
   }
 
   return Return;
 }
 
-void unicorn::MonsterTeleport(const char* NeighMsg)
+void unicorn::MonsterTeleport(cchar* NeighMsg)
 {
   if(CanBeSeenByPlayer())
     ADD_MESSAGE("%s %s and disappears!", CHAR_NAME(DEFINITE), NeighMsg);
@@ -2023,6 +2040,9 @@ void carnivorousplant::GetAICommand()
   if(AttackAdjacentEnemyAI())
     return;
 
+  if(CheckForUsefulItemsOnGround()) 
+    return;
+
   if(MoveRandomly())
     return;
 
@@ -2333,3 +2353,64 @@ void lobhse::CreateCorpse(lsquare* Square)
 {
   largecreature::CreateCorpse(Square);
 }
+
+void mindworm::GetAICommand()
+{
+  character* NeighbourEnemy = GetRandomNeighbour(HOSTILE);
+
+
+  if(NeighbourEnemy && NeighbourEnemy->IsHumanoid() && NeighbourEnemy->HasHead()
+  && !NeighbourEnemy->IsInfectedByMindWorm())
+  {
+    TryToImplantLarvae(NeighbourEnemy);
+    return;
+  }
+
+  character* NearestEnemy = GetNearestEnemy();
+
+  if(NearestEnemy)
+  {
+    PsiAttack(NearestEnemy);
+    return;
+  }
+
+  if(MoveRandomly())
+    return;
+
+  EditAP(-1000);
+}
+
+void mindworm::TryToImplantLarvae(character* Victim)
+{
+  if(Victim->MindWormCanPenetrateSkull(this))
+  {
+    Victim->SetCounterToMindWormHatch(100);
+    if(Victim->IsPlayer())
+    {
+      ADD_MESSAGE("%s penetrates digs through your skull, lays %s eggs and jumps out.", 
+		  CHAR_NAME(DEFINITE), CHAR_POSSESSIVE_PRONOUN);
+    }
+    else if(Victim->CanBeSeenByPlayer())
+    {
+      ADD_MESSAGE("%s penetrates digs through %s's skull, lays %s eggs and jumps out.", 
+		  CHAR_NAME(DEFINITE), Victim->CHAR_NAME(DEFINITE), CHAR_POSSESSIVE_PRONOUN);
+    }
+    MoveRandomly();
+  }
+}
+
+void mindworm::PsiAttack(character* Victim)
+{
+  if(Victim->IsPlayer())
+  {
+    ADD_MESSAGE("Your brain is on fire.");
+  }
+  else if(Victim->CanBeSeenByPlayer() && PLAYER->GetAttribute(PERCEPTION) > RAND_N(20))
+  {
+    ADD_MESSAGE("%s looks scared.", Victim->CHAR_NAME(DEFINITE));
+  }
+
+
+  Victim->ReceiveDamage(this, 1 + RAND_N(5), PSI, ALL, 8, false, false, false, false);
+  Victim->CheckDeath(CONST_S("killed by ") + GetName(INDEFINITE) + "'s psi attack", this);
+}
